"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
// import { callUnomi, resolvePromise, createPromise } from "../utils/connection";
const connection_1 = require("../utils/connection");
const validation_1 = require("../utils/validation");
function resolveReformatSegment(url, headers) {
    return __awaiter(this, void 0, void 0, function* () {
        const callObjectPromise = yield connection_1.callUnomi("GET", url, null, headers, 200);
        if (callObjectPromise.statusCode === 200) {
            return new Promise((resolve, reject) => {
                resolve(validation_1.reformatSegment(callObjectPromise, callObjectPromise.responseData));
            });
        }
        else {
            return new Promise((resolve, reject) => {
                resolve(callObjectPromise);
            });
        }
    });
}
/**
 * @function getAll
 * @param {string} baseUrl
 * @param {Record<string, string>} headers
 * @param {string} sortOrder
 * @param {number} limit
 * @param {number} offset
 * @returns {FilteredResponse}
 */
function getAll(baseUrl, headers, sortOrder, limit, offset) {
    const url = baseUrl + `/cxs/segments?size=${limit}&offset=${offset}&sort=${sortOrder}`;
    return connection_1.callUnomi("GET", url, null, headers, 200);
}
exports.getAll = getAll;
/**
 * @function get
 * @param {string} baseUrl
 * @param {Record<string, string>} headers
 * @param {string} segmentId
 * @returns {FilteredResponse}
 */
function get(baseUrl, headers, segmentId) {
    const url = baseUrl + `/cxs/segments/${segmentId}`;
    return resolveReformatSegment(url, headers);
}
exports.get = get;
/**
 * @function create
 * @param {string} baseUrl
 * @param {Record<string, string>} headers
 * @param {object} properties
 * @returns {FilteredResponse}
 */
function create(baseUrl, headers, params) {
    const url = baseUrl + `/cxs/segments`;
    const validatedSegmentObject = validation_1.validateSegmentObject(params, false);
    if (validatedSegmentObject.areSubConditionsValid) {
        return connection_1.callUnomi("POST", url, validatedSegmentObject.segmentObject, headers, 204);
    }
    else {
        return connection_1.callUnomi("POST", url, validatedSegmentObject.segmentObject, headers, 204, validatedSegmentObject.errorCode);
    }
}
exports.create = create;
/**
 * @function update
 * @param {string} baseUrl
 * @param {Record<string, string>} headers
 * @param {object} properties
 * @returns {FilteredResponse}
 */
function update(baseUrl, headers, params) {
    const url = baseUrl + `/cxs/segments`;
    const validatedSegmentObject = validation_1.validateSegmentObject(params, true);
    if (validatedSegmentObject.areSubConditionsValid) {
        return connection_1.callUnomi("POST", url, validatedSegmentObject.segmentObject, headers, 204);
    }
    else {
        return connection_1.callUnomi("POST", url, validatedSegmentObject.segmentObject, headers, 204, validatedSegmentObject.errorCode);
    }
}
exports.update = update;
/**
 * @function delete
 * @param {string} baseUrl
 * @param {Record<string, string>} headers
 * @param {string} segmentId
 * @returns {FilteredResponse}
 */
function deleteSegment(baseUrl, headers, segmentId) {
    const url = baseUrl + `/cxs/segments/${segmentId}`;
    return connection_1.callUnomi("DELETE", url, null, headers, 200);
}
exports.deleteSegment = deleteSegment;
