"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * @function callUnomi
 * @param {string} method
 * @param {string} url
 * @param {object} body
 * @param {Record<string, string>} headers
 * @param {string} successStatus
 * @returns {FilteredResponse}
 */
function callUnomi(method, url, body, headers, successStatus, errorCode) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            if (errorCode) { // if errorCode is not empty
                return {
                    statusCode: 500,
                    errorMessage: "Something went wrong processing this request. Please try again later or contact support if this error persists. Error Code: " + errorCode,
                    response: {
                        method: method,
                        url: url,
                        body: body,
                        successStatus: successStatus
                    }
                };
            }
            let response;
            if (body === null) {
                console.log(headers);
                response = yield fetch(url, {
                    method: method,
                    headers: headers,
                    credentials: "omit",
                    mode: 'cors',
                });
            }
            else {
                response = yield fetch(url, {
                    method: method,
                    headers: headers,
                    body: JSON.stringify(body),
                    credentials: "omit",
                    mode: 'cors',
                });
            }
            let data = new Object;
            if (response.status != 204) {
                data = yield response.json();
            }
            if (response.status === successStatus) {
                return {
                    statusCode: response.status,
                    responseData: data
                };
            }
            return {
                statusCode: response.status,
                errorMessage: "Something went wrong processing this request. Please try again later or contact support if this error persists. Error Code: 5xU04",
                response: response
            };
        }
        catch (err) {
            return {
                statusCode: 500,
                errorMessage: "Something went wrong processing this request. Please try again later or contact support if this error persists. Error Code: 5xU01",
                response: {
                    method: method,
                    url: url,
                    body: body,
                    successStatus: successStatus
                }
            };
        }
    });
}
exports.callUnomi = callUnomi;
/**
 * @function callElasticsearch
 * @param {string} method
 * @param {string} url
 * @param {object} body
 * @param {string} headers
 * @param {string} successStatus
 * @returns {FilteredResponse}
 */
function callElasticsearch(method, url, headers, successStatus) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            let response;
            response = yield fetch(url, {
                method: method,
                headers: headers,
                credentials: "omit",
                mode: 'cors',
            });
            let data = yield response.json();
            data = data["context-profile"]["mappings"]["properties"]["properties"]["properties"];
            if (response.status === successStatus) {
                return {
                    statusCode: response.status,
                    responseData: Object.keys(data) // list of the keys of the data object
                };
            }
            return {
                statusCode: response.status,
                errorMessage: "Something went wrong processing this request. Please try again later or contact support if this error persists. Error Code: 5xU05",
                response: response
            };
        }
        catch (err) {
            return {
                statusCode: 500,
                errorMessage: "Something went wrong processing this request. Please try again later or contact support if this error persists. Error Code: 5xU06",
                response: {
                    method: method,
                    url: url,
                    successStatus: successStatus
                }
            };
        }
    });
}
exports.callElasticsearch = callElasticsearch;
/**
 * @function callResponse
 * @param {string} method
 * @param {string} url
 * @param {any} body
 * @param {string} successStatus
 * @returns {FilteredResponse}
 */
function callResponse(method, url, body, successStatus) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            return {
                statusCode: successStatus,
                responseData: body // list of the keys of the data object
            };
        }
        catch (err) {
            return {
                statusCode: 500,
                errorMessage: "Something went wrong processing this request. Please try again later or contact support if this error persists. Error Code: 5xU07",
                response: {
                    method: method,
                    url: url,
                    successStatus: successStatus
                }
            };
        }
    });
}
exports.callResponse = callResponse;
