import { callUnomi } from "../utils/connection";
import { FilteredResponse } from "../../src/types/sdkResponse";

/**
 * @function getAll
 * @param {string} baseUrl 
 * @param {Record<string, string>} headers 
 * @returns {FilteredResponse}
 */
export function getAll(baseUrl: string, headers: Record<string, string>): FilteredResponse {
    const url = baseUrl + `/cxs/rules`;

    return callUnomi("GET", url, null, headers, 200);
}

/**
 * @function get
 * @param {string} baseUrl 
 * @param {Record<string, string>} headers 
 * @param {string} ruleId
 * @returns {FilteredResponse}
 */
export function get(baseUrl: string, headers: Record<string, string>, ruleId: string): FilteredResponse {
    const url = baseUrl + `/cxs/rules/${ruleId}`;

    return callUnomi("GET", url, null, headers, 200);
}

/**
 * @function create 
 * @param {string} baseUrl 
 * @param {Record<string, string>} headers 
 * @param {object} properties 
 * @returns {FilteredResponse}
 */
export function create(baseUrl: string, headers: Record<string, string>, properties: object): FilteredResponse {
    const url = baseUrl + `/cxs/rules`;

    return callUnomi("POST", url, properties, headers, 204);
}